package semmiedev.disc_jockey_revive;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.client.util.InputUtil;
import net.minecraft.block.enums.NoteBlockInstrument;
import net.minecraft.text.Text;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

public class KeyMappingManager {
    private static final File MAPPINGS_FILE = new File(FabricLoader.getInstance().getConfigDir().toFile(), "disc_jockey" + "/key_mappings.json");
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();
    private static final Type MAPPING_TYPE = new TypeToken<HashMap<String, NoteData>>() {}.getType();

    private Map<InputUtil.Key, Note> mappings = new HashMap<>();

    private static class NoteData {
        String instrument;
        byte note;

        public NoteData(Note note) {
            this.instrument = note.instrument().name();
            this.note = note.note();
        }

        public Note toNote() {
            try {
                NoteBlockInstrument instrumentEnum = NoteBlockInstrument.valueOf(instrument);
                return new Note(instrumentEnum, note);
            } catch (IllegalArgumentException e) {
                Main.LOGGER.error("键位映射中出现未知乐器 '{}' ", instrument);
                return null;
            }
        }
    }

    public KeyMappingManager() {
        loadMappings();
    }

    public void loadMappings() {
        if (!MAPPINGS_FILE.exists()) {
            Main.LOGGER.info("未找到键映射文件，正在创建默认的类似于 FL Studio 的映射。");
            mappings.clear();
            
            NoteBlockInstrument dirt = NoteBlockInstrument.HARP;
            NoteBlockInstrument wood = NoteBlockInstrument.BASS;
            NoteBlockInstrument gold = NoteBlockInstrument.BELL;
            
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_Z, 0), new Note(wood, (byte) 6));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_S, 0), new Note(wood, (byte) 7));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_X, 0), new Note(wood, (byte) 8));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_D, 0), new Note(wood, (byte) 9));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_C, 0), new Note(wood, (byte) 10));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_V, 0), new Note(wood, (byte) 11));

            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_G, 0), new Note(dirt, (byte) 0));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_B, 0), new Note(dirt, (byte) 1));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_H, 0), new Note(dirt, (byte) 2));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_N, 0), new Note(dirt, (byte) 3));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_J, 0), new Note(dirt, (byte) 4));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_M, 0), new Note(dirt, (byte) 5));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_COMMA, 0), new Note(dirt, (byte) 6));

            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_Q, 0), new Note(dirt, (byte) 6));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_2, 0), new Note(dirt, (byte) 7));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_W, 0), new Note(dirt, (byte) 8));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_3, 0), new Note(dirt, (byte) 9));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_E, 0), new Note(dirt, (byte) 10));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_R, 0), new Note(dirt, (byte) 11));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_5, 0), new Note(dirt, (byte) 12));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_T, 0), new Note(dirt, (byte) 13));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_6, 0), new Note(dirt, (byte) 14));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_Y, 0), new Note(dirt, (byte) 15));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_7, 0), new Note(dirt, (byte) 16));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_U, 0), new Note(dirt, (byte) 17));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_I, 0), new Note(dirt, (byte) 18));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_9, 0), new Note(dirt, (byte) 19));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_O, 0), new Note(dirt, (byte) 20));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_0, 0), new Note(dirt, (byte) 21));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_P, 0), new Note(dirt, (byte) 22));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_LEFT_BRACKET, 0), new Note(dirt, (byte) 23));
            mappings.put(InputUtil.fromKeyCode(InputUtil.GLFW_KEY_EQUAL, 0), new Note(dirt, (byte) 24));


            saveMappings();
            return;
        }

        try (FileReader reader = new FileReader(MAPPINGS_FILE)) {
            HashMap<String, NoteData> loadedData = GSON.fromJson(reader, MAPPING_TYPE);
            mappings.clear();
            if (loadedData != null) {
                for (Map.Entry<String, NoteData> entry : loadedData.entrySet()) {
                    InputUtil.Key key = InputUtil.fromTranslationKey(entry.getKey());
                    Note note = entry.getValue().toNote();
                    if (key != InputUtil.UNKNOWN_KEY && note != null) {
                        mappings.put(key, note);
                    }
                }
            }
            Main.LOGGER.info("已加载按键映射。");
        } catch (IOException e) {
            Main.LOGGER.error("加载按键映射失败。", e);
        }
    }



    public void saveMappings() {
        MAPPINGS_FILE.getParentFile().mkdirs();
        try (FileWriter writer = new FileWriter(MAPPINGS_FILE)) {
            HashMap<String, NoteData> dataToSave = new HashMap<>();
            for (Map.Entry<InputUtil.Key, Note> entry : mappings.entrySet()) {
                dataToSave.put(entry.getKey().getTranslationKey(), new NoteData(entry.getValue()));
            }
            GSON.toJson(dataToSave, writer);
            Main.LOGGER.info("已保存按键映射。");
        } catch (IOException e) {
            Main.LOGGER.error("保存按键映射失败。", e);
        }
    }

    public Note getNoteForKey(InputUtil.Key key) {
        return mappings.get(key);
    }

    public void setMapping(InputUtil.Key key, Note note) {
        mappings.put(key, note);
    }

    public void removeMapping(InputUtil.Key key) {
        mappings.remove(key);
    }

    public Map<InputUtil.Key, Note> getMappings() {
        return new HashMap<>(mappings);
    }

    public static String getNoteDisplayName(Note note) {
        if (note == null) return "未设置";

        String instrumentTranslationKey = "block.minecraft.note_block.instrument." + note.instrument().asString();
        String instrumentName = Text.translatable(instrumentTranslationKey).getString();

        int pitch = note.note();
        String[] noteNames = {"F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#", "E", "F"};

        int octave;
        if (pitch >= 19) {
            octave = 1;
        } else if (pitch >= 7) {
            octave = 0;
        } else {
            octave = -1;
        }

        int noteIndexInArray = (pitch - 7 + 12 * 2) % 12;
        String noteName = noteNames[noteIndexInArray];

        return instrumentName + " " + noteName + "(" + octave + ")";
    }
}
